#include <libucdaemon.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>

int
parent_init(int argc, char *argv[])
{
	pid_t pid;

	ucd_log(LOG_INFO, "do some stuff before daemonizing");

	pid = ucd_pidfile_read(NULL);
	if (pid != -1) {
		ucd_log(LOG_INFO, "another daemon process is active: %d", pid);
		return -1;
	}

	ucd_daemon_wait(10);
	return 0;
}

int
daemon_main(int argc, char *argv[])
{
	int i;

	ucd_log(LOG_INFO, "initialize daemon");
	ucd_pidfile_lock(NULL);

	if (argc > 1) {
		if (strcmp(argv[1], "--simulate-failure") == 0) {
			ucd_error("failed to initialize daemon");
		} else if (strcmp(argv[1], "--simulate-segfault") == 0) {
			*((int *) -1) = 0;
		} else if (strcmp(argv[1], "--simulate-timeout") == 0) {
			sleep(20);
		}
	}

	if (argc > 1 && strcmp(argv[1], "--signal-status") == 0) {
		ucd_daemon_signal_status(atoi(argv[2]));
	} else {
		ucd_daemon_signal();
	}

	for (i = 0; i < 10;  i++) {
		ucd_log(LOG_INFO, "daemon is running (%d)", i);
		sleep(5);
	}

	ucd_log(LOG_INFO, "finish daemon");
	ucd_pidfile_unlock();

	return 0;
}

int
main(int argc, char *argv[])
{
	int flags = 0;

	if (argc > 1 && strcmp(argv[argc - 1], "-D") == 0) {
		flags = UCD_NODETACH;
		argv[--argc] = NULL;
	}

	return ucd_daemon_start(NULL, flags,
				parent_init, daemon_main,
				argc, argv);
}
